//
// $Id: ExternalVariable.cpp,v 1.6 2006/10/27 18:35:17 bakerj Exp $
//
//****************************************************************************************//
// Copyright (c) 2006, The MITRE Corporation
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice, this list
//       of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright notice, this 
//       list of conditions and the following disclaimer in the documentation and/or other
//       materials provided with the distribution.
//     * Neither the name of The MITRE Corporation nor the names of its contributors may be
//       used to endorse or promote products derived from this software without specific 
//       prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
// SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
// OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//****************************************************************************************//

#include "ExternalVariable.h"

//****************************************************************************************//
//									ExternalVariable Class								  //	
//****************************************************************************************//

ExternalVariable::ExternalVariable(string id, string name, int version, OvalEnum::Datatype datatype, StringVector* msgs) : AbsVariable (id, name, version, datatype, msgs) {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	//	Create a compelete ExternalVariable
	//
	// -----------------------------------------------------------------------

}

ExternalVariable::~ExternalVariable() {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	// -----------------------------------------------------------------------

}

// ***************************************************************************************	//
//								 Public members												//
// ***************************************************************************************	//
void ExternalVariable::Parse(DOMElement* externalVariableElm) {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	//	Parse the provided ExternalVariable element into a ExternalVariable
	//
	// -----------------------------------------------------------------------

	this->SetId(XmlCommon::GetAttributeByName(externalVariableElm, "id"));
	this->SetDatatype(OvalEnum::ToDatatype(XmlCommon::GetAttributeByName(externalVariableElm, "datatype")));
	string versionStr = XmlCommon::GetAttributeByName(externalVariableElm, "version");
	int version = atoi(versionStr.c_str());
	this->SetVersion(version);

	// Get all the possible elements' values
	DOMNodeList *externalVariableElmChildren = externalVariableElm->getChildNodes();
	unsigned int index = 0;
	while(index < externalVariableElmChildren->getLength()) {
		DOMNode *tmpNode = externalVariableElmChildren->item(index);
		if (tmpNode->getNodeType() == DOMNode::ELEMENT_NODE) {
			DOMElement *childElm = (DOMElement*)tmpNode;
			string elmName = XmlCommon::GetElementName(childElm);
			
			// parse each possible element
			if(elmName.compare("possible") == 0) {
				PossibleType *possibleType = new PossibleType();
				possibleType->Parse(childElm);
				this->AppendPossibleType(possibleType);
			}
		}
		index ++;
	}

	// Finally call ComputeValue
	this->ComputeValue();
}

VariableValueVector* ExternalVariable::GetVariableValues() {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	//	return the variable values used to compute this variable's value
	//	in this case just an empty vector.
	// -----------------------------------------------------------------------
	
	VariableValueVector* values = new VariableValueVector();

	return values;
}

PossibleTypeVector* ExternalVariable::GetPossibleTypes() {
	return &this->possibleTypes;
}

void ExternalVariable::AppendPossibleType(PossibleType* pt) {
	this->possibleTypes.push_back(pt);
}

void ExternalVariable::ComputeValue() {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	//	Fetch the variable from the external-variables.xml file.
	//	Then get each value associated with the variable.
	//	Then validate each value then create a VariableValue.
	//	
	// -----------------------------------------------------------------------

	// get the external variables file
	XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument* externalVariableDoc = NULL;
	try {
		externalVariableDoc = DocumentManager::GetExternalVariableDocument();
	} catch(Exception ex) {
		this->SetFlag(OvalEnum::ERROR_FLAG);
		this->AppendMessage("Error parsing external variable file. " + ex.GetErrorMessage());
		return;
	} catch(...) {
		this->SetFlag(OvalEnum::ERROR_FLAG);
		this->AppendMessage("Unknown error parsing external variable file.");
        return;
	}

	// get the variable from the external variables file
	DOMElement* variableElm = XmlCommon::FindElementNS(externalVariableDoc, "variable", "id", this->GetId());
	if(variableElm == NULL) {
		this->SetFlag(OvalEnum::ERROR_FLAG);
		this->AppendMessage("Error unable to find external variable " + this->GetId() + " in external variable document.");
	} else {

		// get each value from the variable element in the external variables document.
		DOMNodeList *variableElmChildren = variableElm->getChildNodes();
		unsigned int index = 0;
		while(index < variableElmChildren->getLength()) {
			DOMNode *tmpNode = variableElmChildren->item(index);
			if (tmpNode->getNodeType() == DOMNode::ELEMENT_NODE) {
				DOMElement *childElm = (DOMElement*)tmpNode;
				string elmName = XmlCommon::GetElementName(childElm);
				
				// parse each value element
				if(elmName.compare("value") == 0) {
					string value = XmlCommon::GetDataNodeValue(childElm);

					// validate the value
					if(this->ValidateValue(this->GetDatatype(), value)) {

						// add the value to the set of values for this exteranl variable.
						VariableValue* varValue = new VariableValue(this->GetId(), value);
						this->AppendVariableValue(varValue);

					} else {
						this->SetFlag(OvalEnum::ERROR_FLAG);
						this->AppendMessage("Error a value ('" + value + "') of external variable " + this->GetId() + " does not match the possible values for the variable.");
						return;
					}
				}
			}
			index ++;
		}
		this->SetFlag(OvalEnum::COMPLETE_FLAG);
	}

}

bool ExternalVariable::ValidateValue(OvalEnum::Datatype datatype, string value) {
	// -----------------------------------------------------------------------
	//	Abstract
	//
	//	Ensure that the specified value matches the criteria 
	//	specified by the possible elements of this variable
	//	
	// ----------------------------------------------------------------------

	bool isValid = false;
	
	// loop through all possible elements - if any are true return true 
	PossibleTypeVector::iterator iterator;
	for(iterator = this->GetPossibleTypes()->begin(); iterator != this->GetPossibleTypes()->end(); iterator++) {
		isValid = (*iterator)->ValidateValue(datatype, value);
		if(isValid) {
			break;
		}
	}

	// if no possible elements then no restrictions on the value so return true.
	if(this->GetPossibleTypes()->size() == 0) {
		isValid = true;
	}

	return isValid;
}
